.TH INVESTIGATION 1 "24 August 2013"
.SH NAME
\fBinvestigation\fP \- starts an http server to ease investigations
.SH SYNOPSIS
investigation [-H host] [-P port] [-d] [-v] [-h] file
.SH DESCRIPTION
This command will parse the given file (or standard input if the parameter "-" is provided instead), build the investigation site and starts a small HTTP server to start investigations.
.SH OPTIONS
.TP
.B -H host
This option restricts the origin of the queries on the HTTP server.
.TP
.B -P port
This option specify on which port the HTTP server will listen.
.TP
.B -d
This option switches to debug mode. The server will not detach from the terminal, and will display some useful information for site debugging.
.TP
.B -s
This option switches to site profiling mode. The server is not launched in this mode. An investigation is made on the site, and statistics about pages, conditions and identifiers is displayed instead.
.TP
.B -h
This option displays the basic help on the investigation command.
.TP
.B -v
This option displays the version and licence of the investigation command.
.SH SITE NAVIGATION
When the web server starts, it accepts HTTP GET requests on the / page or the /investigation.html page with the parameter q and a hash code.
When the / page or an invalid or inexistant hash code is given to the server, it will return the startup page of the investigation.
The page served is made of two main parts:
.IP "The first one is the current state of the investigation, with all the sub pages displayed,"
.IP "The second one is the history of the investigation, with a link by answered question. Each line contains the name of the sub page, the question and the provided answer."
.P
At each step of the investigation, the user can click on the link of a reply, and the server will reply with the next step of the investigation.
The URI with the hash code is valid until the server shuts down.
If the server is restarted, all links are obsoletes, and may be redirected to the startup page.
It should prevent a link found after an investigation to refer to a wrong page after reload of the server with a different investigation file.
.P
Internally, each hash code is linked to a sequence of identifiers defined in the investigation file.
This sequence is used to display some sub pages among the ones defined in the investigation file.
In debug mode, the web server remains attached to the terminal, and log each received query, with the hash code and the sequence of identifiers.
.SH FILE SYNTAX
The input file has its own syntax, and describes the complete investigation site. In this section,
.IP "Keywords are words in capital letters,"
.IP "Identifiers are words in small letters or digits or underscores, but always starts with a letter,"
.IP "Strings are characters between double quotes, without double quote or newline,"
.IP "Text lines are character lines starting with a dot not preceded by whitespaces."
.SS "Global structure"
The first instruction shall be the title of the site.
Then, the startup page shall be defined. See below for the definition of a page.
After the startup page, other pages or conditional structures can be added. Their syntax are explained after.
At the end of the file, the style bloc may be defined.
.SS Site title
The title is set with this syntax:
.nf
	\fBTITLE\fP "Site title"
.fi
.SS "Page definition"
A page definition starts with the keyword \fBPAGE\fP followed by a list of identifier.
.P
Then, an optional title can be added this the syntax seen above.
.P
The body of the page is a sequence of text blocs and questions:
.IP "Text blocks starts with the keyword \fBTEXT\fP and ends with the keyword \fBEND\fP. Between those two keywords, a sequence of text lines can be added."
.IP "Questions starts with the keyword \fBQUESTION\fP followed by a string containing the question, and ends with the keyword \fBEND\fP. Before the last keyword, a sequence of answer can be specified. Each answer is a made of an identifier followed by a string containing the answer. The identifier shall correspond to a page identifier. Conditional structures can be specified in the answer list."
.P
The page definition ends with the keyword \fBEND\fP.
.nf
	\fBPAGE\fP page
		\fBTITLE\fP "Page title"
		\fBTEXT\fP
	.A text line,
	.a second text line.
		\fBEND\fP
		\fBQUESTION\fP "A simple question?"
			yes "Yes"
			no "No"
			\fBCONDITIONAL\fP
			\fBCASE\fP page2 \fIAND\fP \fINOT\fP page3 \fBTHEN\fP
				maybe "Maybe"
				mainpage "Return to main page"
			\fBEND\fP
		\fBEND\fP
	\fBEND\fP
.fi
When the keyword \fBPAGE\fP follows the \fBDRAFT\fP keyword, the generation of the HTML page adds the "draft" class, to let the CSS render the page in another way.
.SS "Conditionals"
A conditional structure starts with the keyword \fBCONDITIONAL\fP and is followed by conditional clauses made of a condition and a sequence of sub conditional structures or elements that the current conditional structure replaces, and ends with the \fBEND\fP keyword.
Each clause starts with the keyword \fBCASE\fP followed by a condition terminated with the keyword \fBTHEN\fP. After this keyword, the sequence of instructions that need to be activated when the condition is true can be placed, until the next clause.
The last clause can start with the keyword \fBELSE\fP instead of the condition and the \fBCASE\fP and \fBTHEN\fP keywords. This last clause will be activated when no clauses with a condition has been activated.
In a conditional structure, the first clause triggered will shadow the following clauses.
.SS "Conditions"
Each query made to the server is linked to a sequence of identifiers.
When the sequence is empty, the startup page is displayed.
Otherwise, the last identifier is an implicit condition that will trigger all pages defined with this identifier.
When several identifiers are defined on a page, the page is activated if the last identifier is among the set of the identifiers linked to the page.
For some queries, testing the presence or the absence of an identifier can be important to avoid wrong tracks during the investigation.
.P
In a condition, putting an identifier will return TRUE when the identifier is in the query identifier list, and FALSE otherwise.
.P
Pattern conditions can be specified between \fB~\fP characters, where the result is TRUE when the whole query matches the given pattern:
.IP "an identifer matches itself"
.IP "a dot matches one identifier: \fB.\fP"
.IP "a sub pattern is surrounded by parenthesis: \fB(\fP sub pattern \fB)\fP"
.IP "a conditional sub pattern is followed by a question mark: sub pattern \fB?\fP"
.IP "an optional repetition of a sub pattern is followed by a star: sub pattern \fB*\fP"
.IP "a mandatory repetition of a sub pattern is followed by a plus: sub pattern \fB+\fP"
.IP "an alternative of sub patterns is surrounded by parenthesis, each sub pattern is separated from others by a pipe: \fB(\fP sub pattern 1 \fB|\fP sub pattern 2 \fB|\fP ... \fB|\fP sub pattern last \fB)\fP"
When the first \fB~\fP character is preceded by a question mark, the operators are considered as non greedy. By default, the operators are greedy, and will match all possibilities.
.P
Atomic conditions can be gathered into more complex conditions using conditional operators:
.IP "\fINOT\fP followed by an atomic condition will negates the atomic condition,"
.IP "\fINOT\fP followed by a conditional operator in parenthesis will negates the result of the operator,"
.IP "Other operators are infixed with a keyword between each sub condition, and without parenthesis except for sub conditions that are another conditional operator in \fIAND\fP, \fIOR\fP and \fIXOR\fP,"
.IP "\fIAND\fP operator is TRUE when all sub conditions are TRUE, and is FALSE otherwise,"
.IP "\fIOR\fP operator is TRUE when at least one sub condition is TRUE, and is FALSE otherwise,"
.IP "\fIXOR\fP operator is TRUE when only one sub condition is TRUE, and is FALSE otherwise."

.nf
	Examples of valid conditions:
	id
	\fINOT\fP id
	\fINOT\fP id \fIAND\fP id2 \fIAND\fP (id3 \fIOR\fP id4) \fIAND\fP \fINOT\fP (id1 \fIXOR\fP id5 \fIXOR\fP id3)
	id \fIAND\fP \fB~ . * id ( id id3 | id2 id4 | id3 . ) + . * id4 ~\fP \fIAND\fP id5
.fi

.SS "Site style"
At the end of the site file, a style section can be defined. This section starts with the keyword \fBSTYLE\fP and ends with the keyword \fBEND\fP. The body of this section is text lines containing the CSS stylesheet.
.SS "A complete example"
An example of an investigation file can be found in the file ===PKGDATADIR===/example.inv and this file can be used to check how the investigation site works.
.SH CONDITION DEBUGGING
Having conditions working correctly can be a long pain.
To avoid this, a special mode can be used to check different conditions on queries.
This special mode can be activated by commands in a file, but this mode is optimal in an interactive way.
Please note that the first \fBDEBUG\fP command shall be syntaxicaly correct to enter this mode.
After the first \fBEND\fP, syntax errors are permitted, and will be shown when a \fBEND\fP, a \fBTHEN\fP or a \fB~\fP is entered.
.P
The commands are:
.IP "\fBDEBUG\fP list of identifiers \fBEND\fP: set a new request used to check conditions and regular expressions,"
.IP "\fBCASE\fP condition \fBTHEN\fP: check whether the current request makes this condition TRUE or FALSE,"
.IP "\fB~\fP regular expression \fB~\fP: check whether the current request matches the regular expression in greedy mode and non greedy mode."
.P
To exit the debug mode, a control-d should close the command line.
.SH RETURN VALUES
The server returns 0 in case of success, and a non zero value otherwise.
.SH AUTHOR
This software is developed and maintained by Julien Bruguier <projet.investigation@pappy.tf>
.SH BUGS
There are no bugs. Only features... ;-)
