
/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_ARBREINVESTIGATION__
#define __INVESTIGATION_DONNEES_ARBREINVESTIGATION__

#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(ArbreInvestigation); DECL_WEAK_PTR(ArbreInvestigation); } }
#include<src/investigation/donnees/requete.h>
#include<src/investigation/donnees/site.h>
#include<src/investigation/donnees/compteurs.h>

namespace Investigation
{
	namespace Donnees
	{
		class InvestigationSite;
		class ArbreInvestigation : public std::enable_shared_from_this<ArbreInvestigation>
		{
			friend class InvestigationSite;
			public:
				ArbreInvestigation(const Requete& requete, const ArbreInvestigationSP& pere = ArbreInvestigationSP())
				:_requete(requete), _pere(pere) {};
				ArbreInvestigationSP operator+= (const Requete& requete)
				{
					ArbreInvestigationSP arbre(new ArbreInvestigation(requete,this->shared_from_this()));
					_fils[requete] = arbre;
					return arbre;
				};
				bool operator%(const SiteSP& site) const
				{
					for(ArbreInvestigationCSP noeud_courant=this->shared_from_this() ; noeud_courant ; noeud_courant=noeud_courant->_pere.lock())
					{	
						if((noeud_courant->_site) and (*(noeud_courant->_site))%(*site))
						{
#ifdef INVESTIGATION_DEBUG
	std::cout << "O" << std::endl;
#endif
							return true;
						}
#ifdef INVESTIGATION_DEBUG
	std::cout << ".";
#endif
					}
#ifdef INVESTIGATION_DEBUG
	std::cout << "X" << std::endl;
#endif
					return false;
				}
				CompteursSite compte() const
				{
					CompteursSite compteurs;
					compte(compteurs);
					return compteurs;
				};
			private:
				Requete _requete;
				SiteSP _site;
				std::map<std::string,ArbreInvestigationSP> _fils;
				ArbreInvestigationWP _pere;
				void compte(CompteursSite& compteurs) const
				{
					if(_site)
						_site->compte(compteurs);
					for(auto it=_fils.cbegin() ; it!=_fils.cend() ; ++it)
					{
						it->second->compte(compteurs);
					}
				}
		};
	}
}
#endif
