/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_LIENS__
#define __INVESTIGATION_DONNEES_LIENS__

#include <map>
#include <vector>
#include <algorithm>
#include <sstream>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Liens); } }
#include <src/global/exceptions.h>
#include <src/investigation/donnees/requete.h>

namespace Investigation
{
	namespace Donnees
	{
		class PlusDeLiensDisponibles : public Investigation::Global::Exception::Execution
		{
			public:
				PlusDeLiensDisponibles()
				:Execution(SIGTERM,"No more link available. Server must be relaunched") { };
		};

		class Liens
		{
			public:
				Liens()
				:_compteur(0), _debut_compteur(0)
				{
					::srand(::time(NULL));
					_debut_compteur = ::rand();
					_compteur = _debut_compteur;
				};
				std::string operator[] (const Investigation::Donnees::Requete& requete)
				{
					for(auto it=_liens.cbegin() ; it!=_liens.cend() ; ++it)
					{
						if(it->second==requete)
						{
							return it->first;
						}
					}
					std::string lien_court = Liens::lien_court(_compteur++);
					if(_compteur==_debut_compteur)
					{
						throw PlusDeLiensDisponibles();
					}
					_liens[lien_court]=requete;
					return lien_court;

				};
				Investigation::Donnees::Requete operator[](const std::string& lien) const
				{
					auto it=_liens.find(lien);
					if(it==_liens.end())
					{
						return Investigation::Donnees::Requete();
					}
					return it->second;
				};
			private:
				unsigned long long int _compteur;
				unsigned long long int _debut_compteur;
				std::map<std::string,Investigation::Donnees::Requete> _liens;
				static std::string lien_court(unsigned long long int compteur)
				{
					if(compteur==0)
					{
						return "0";
					}
					std::ostringstream resultat;
					while(compteur>0)
					{
						size_t chiffre = compteur%36;
						if(chiffre<10)
						{
							resultat << static_cast<char>('0'+chiffre);
						}
						else
						{
							resultat << static_cast<char>('A'+(chiffre-10));
						}
						compteur /= 36;
					}
					std::string lien = resultat.str();
					return std::string(lien.rbegin(),lien.rend());
				}
		};
	}
}
#endif
