/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#ifndef __INVESTIGATION_DONNEES_PAGE__
#define __INVESTIGATION_DONNEES_PAGE__

#include <set>
#include <vector>
#include <src/global/global.h>
namespace Investigation { namespace Donnees { DECL_SHARED_PTR(Page); DECL_SHARED_PTR(ContenuPage); typedef std::vector<ContenuPageSP> SequenceContenuPage; } }
#include <src/global/sources/position.h>

#include <src/investigation/donnees/html.h>
#include <src/investigation/donnees/instruction.h>
#include <src/investigation/donnees/statistiques.h>
#include <src/investigation/donnees/identifiants.h>

namespace Investigation
{
	namespace Donnees
	{
		class CompteursSite;
		class Requete;
		class Liens;
		class ContenuPage
		{
			public:
				virtual ~ContenuPage() {};
				virtual Html::SequenceContenuDiv operator() (const std::string& methode, const Requete& requete, Liens& liens) const = 0;
				virtual std::set<std::string> operator[] (const Requete& requete) { return std::set<std::string>(); };
				virtual ContenuPageSP clone() const = 0;
				virtual bool operator%(const ContenuPage& contenu_page) const = 0;
				virtual void liens_valides(const std::set<std::string>& identifiants) const = 0;
				virtual void propage_infos(const std::string& page) = 0;
				virtual long int niveau_information() const = 0;
				virtual void compte(CompteursSite& compteurs) const = 0;
		};

		typedef std::vector<PageSP> SequencePages;
		class Page : public Instruction
		{
			friend class CompteursSite;
			public:
				Page(const bool brouillon, const std::vector<std::string>& identifiants, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position);
				Page(const bool brouillon, const std::vector<std::string>& identifiants, const std::string& titre, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position);
				Page(const bool brouillon, const std::set<std::string>& identifiants, const std::string& titre, const SequenceContenuPage& contenu, const Investigation::Global::Source::PositionSP& position, const Identifiant& identifiant);
				virtual ~Page();
				virtual SequencePages operator() (const Requete& requete) const ;
				virtual InstructionSP clone() const ;
				virtual std::set<std::string> operator[] (const Requete& requete) ;
				virtual bool operator%(const Instruction& instruction) const ;
				bool operator%(const Page& page) const ;
				Html::Div operator() (const std::string& methode, const Requete& requete, Liens& liens) const ;
				virtual void liens_valides(const std::set<std::string>& identifiants) const ;
				virtual std::set<std::string> identifiants() const ;
				virtual void propage_infos() ;
				virtual void compte(CompteursSite& compteurs) const ;
				long int niveau_information() const ;
			private:
				bool _brouillon;
				std::set<std::string> _identifiants;
				std::string _titre;
				SequenceContenuPage _contenu;
			public:
				Statistiques _statistiques;
				Identifiant _identifiant;
		};
	}
}
#endif
