/* 
 *  This program is an interactive FAQ to ease investigations.
 *  Copyright (C) 2012 Julien Bruguier.
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */ 

#include <src/investigation/donnees/question.h>

using namespace Investigation::Donnees;

Question::Question(const std::string& question, const SequenceReponses& reponses)
	:_question(question), _reponses(reponses) { }
Question::~Question() {}
Html::SequenceContenuDiv Question::operator() (const std::string& methode, const Requete& requete, Liens& liens) const
{
	Html::SequenceContenuDiv seq;
	Html::P question("investigation page question");
	question << Html::Texte(_question);
	seq.push_back(question);
	Html::Ul liste_reponses("investigation page response");
	for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
	{
		liste_reponses << (**it)(methode,requete,liens);
	}
	seq.push_back(liste_reponses);
	return seq;
}
ContenuPageSP Question::clone() const
{
	SequenceReponses reponses;
	for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
	{
		reponses.push_back((*it)->clone());
	}
	ContenuPageSP contenu(new Question(_question,reponses));
	return contenu;
}
std::set<std::string> Question::operator[] (const Requete& requete)
{
	std::set<std::string> mc;
	for(auto it=_reponses.begin() ; it!=_reponses.end() ; ++it)
	{
		auto itmc = (**it)[requete];
		mc.insert(itmc.cbegin(),itmc.cend());
	}
	return mc;
}
bool Question::operator%(const ContenuPage& contenu_page) const
{
	const Question& question=dynamic_cast<const Question&>(contenu_page); // throw bad_cast
	return (*this)%(question);
}
bool Question::operator%(const Question& question) const
{
	if(_reponses.size()!=question._reponses.size())
		throw;
	for(auto it=_reponses.cbegin(), itr=question._reponses.cbegin() ; it!=_reponses.cend() ; ++it, ++itr)
	{
		if(not ((**it)%(**itr)))
			return false;
	}
	return true;
}
void Question::liens_valides(const std::set<std::string>& identifiants) const
{
	for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
	{
		(*it)->liens_valides(identifiants);
	}
}
void Question::propage_infos(const std::string& page)
{
	for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
	{
		(*it)->propage_infos(page,_question);
	}
}
long int Question::niveau_information() const
{
	return -1;
}
void Question::compte(CompteursSite& compteurs) const
{
	for(auto it=_reponses.cbegin() ; it!=_reponses.cend() ; ++it)
	{
		(*it)->compte(compteurs);
	}
}
